@@ -16,10 +16,11 @@ def main():
 
         scalene_profiler.Scalene.main()
     except Exception as exc:
-        sys.stderr.write("ERROR: Calling scalene main function failed: %s\n" % exc)
+        sys.stderr.write("ERROR: Calling Scalene main function failed: %s\n" % exc)
         traceback.print_exc()
         sys.exit(1)
 
 
 if __name__ == "__main__":
     main()
+
