@@ -7,7 +7,14 @@ from typing import Any
 
 @Scalene.shim
 def replacement_signal_fns(scalene: Scalene) -> None:
-
+    scalene_signals = scalene.get_signals()
+    expected_handlers_map = {
+            scalene_signals.malloc_signal: scalene.malloc_signal_handler,
+            scalene_signals.free_signal: scalene.free_signal_handler,
+            scalene_signals.memcpy_signal: scalene.memcpy_signal_handler,
+            signal.SIGTERM: scalene.term_signal_handler,
+            scalene_signals.cpu_signal: scalene.cpu_signal_handler,
+        }
     old_signal = signal.signal
     if sys.version_info < (3, 8):
 
@@ -28,6 +35,7 @@ def replacement_signal_fns(scalene: Scalene) -> None:
         all_signals = scalene.get_all_signals_set()
         timer_signal, cpu_signal = scalene.get_timer_signals()
         timer_signal_str = signal.strsignal(signum)
+        start_signal, stop_signal = scalene.get_lifecycle_signals()
         if signum == cpu_signal:
             print(
                 f"WARNING: Scalene uses {timer_signal_str} to profile.\n"
@@ -35,6 +43,28 @@ def replacement_signal_fns(scalene: Scalene) -> None:
                 "Code that raises signals from within Python code will be rerouted."
             )
             return old_signal(new_cpu_signal, handler)
+        if signum in [start_signal, stop_signal]:
+            if not scalene.get_lifecycle_disabled():
+                print(
+                    f"WARNING: signal {signal.strsignal(signum)} is used to\n"
+                    "enable or disable Scalene. Starting or stopping Scalene with\n"
+                    "`--start` or `--stop` will be disabled."
+                )
+                # Disable the other signal
+                if signum == start_signal:
+                    old_signal(stop_signal, signal.SIG_IGN)
+                if signum == stop_signal:
+                    old_signal(start_signal, signal.SIG_IGN)
+                scalene.disable_lifecycle()
+            return old_signal(signum, handler)
+        # Fallthrough condition-- if we haven't dealt with the signal at this point in the call and the handler is
+        # a NOP-like, then we can ignore it. It can't have been set already, and the expected return value is the
+        # previous handler, so this behavior is reasonable
+        if signal in all_signals and (handler is signal.SIG_IGN or handler is signal.SIG_DFL):
+            return handler
+        # If trying to "reset" to a handler that we already set it to, ignore
+        if signum in expected_handlers_map and expected_handlers_map[signum] is handler:
+            return signal.SIG_IGN
         if signum in all_signals:
             print(
                 "Error: Scalene cannot profile your program because it (or one of its packages)\n"
