@@ -7,8 +7,7 @@ from __future__ import annotations # work around Python 3.8 issue, see https://s
     See the paper "docs/osdi23-berger.pdf" in this repository for technical
     details on Scalene's design.
 
-    by Emery Berger
-    https://emeryberger.com
+    by Emery Berger, Sam Stern, and Juan Altmayer Pizzorno
 
     usage: scalene test/testme.py
     usage help: scalene --help
@@ -40,6 +39,7 @@ import queue
 import re
 import signal
 import stat
+import subprocess
 import sys
 import sysconfig
 import tempfile
@@ -75,6 +75,7 @@ from typing import (
     cast,
 )
 
+import scalene.scalene_config
 from scalene.scalene_arguments import ScaleneArguments
 from scalene.scalene_client_timer import ScaleneClientTimer
 from scalene.scalene_funcutils import ScaleneFuncUtils
@@ -91,7 +92,6 @@ from scalene.scalene_statistics import (
     ScaleneStatistics,
 )
 from scalene.scalene_utility import *
-from scalene.scalene_version import scalene_version, scalene_date
 
 if sys.platform != "win32":
     import resource
@@ -115,8 +115,6 @@ console.log = nada  # type: ignore
 MINIMUM_PYTHON_VERSION_MAJOR = 3
 MINIMUM_PYTHON_VERSION_MINOR = 8
 
-SCALENE_PORT = 11235
-
 def require_python(version: Tuple[int, int]) -> None:
     assert (
         sys.version_info >= version
@@ -142,10 +140,6 @@ def scalene_redirect_profile(func: Any) -> Any:
 
 builtins.profile = scalene_redirect_profile  # type: ignore
 
-# Must equal src/include/sampleheap.hpp NEWLINE *minus 1*
-NEWLINE_TRIGGER_LENGTH = 98820  # SampleHeap<...>::NEWLINE-1
-
-
 def start() -> None:
     """Start profiling."""
     Scalene.start()
@@ -176,7 +170,7 @@ class Scalene:
     __parent_pid = -1
     __initialized: bool = False
     __last_profiled = [Filename("NADA"), LineNumber(0), ByteCodeIndex(0)]
-
+    __orig_python = sys.executable # will be rewritten later
     @staticmethod
     def last_profiled_tuple() -> Tuple[Filename, LineNumber, ByteCodeIndex]:
         """Helper function to type last profiled information."""
@@ -227,7 +221,9 @@ class Scalene:
     def get_original_lock() -> threading.Lock:
         """Return the true lock, which we shim in replacement_lock.py."""
         return Scalene.__original_lock()
-
+    @staticmethod
+    def get_signals():
+        return Scalene.__signals
     # when did we last receive a signal?
     __last_signal_time = TimeInfo()
 
@@ -271,6 +267,7 @@ class Scalene:
     __orig_signal = signal.signal
     __orig_exit = os._exit
     __orig_raise_signal = signal.raise_signal
+    __lifecycle_disabled = False
 
     __orig_kill = os.kill
     if sys.platform != "win32":
@@ -284,6 +281,16 @@ class Scalene:
         Used by replacement_signal_fns.py to shim signals used by the client program.
         """
         return set(Scalene.__signals.get_all_signals())
+    @staticmethod
+    def get_lifecycle_signals() -> Tuple[signal.Signals, signal.Signals]:
+        return Scalene.__signals.get_lifecycle_signals()
+
+    @staticmethod
+    def disable_lifecycle():
+        Scalene.__lifecycle_disabled = True
+    @staticmethod
+    def get_lifecycle_disabled() -> bool:
+        return Scalene.__lifecycle_disabled
 
     @staticmethod
     def get_timer_signals() -> Tuple[int, signal.Signals]:
@@ -319,7 +326,7 @@ class Scalene:
     @staticmethod
     def update_line() -> None:
         """Mark a new line by allocating the trigger number of bytes."""
-        bytearray(NEWLINE_TRIGGER_LENGTH)
+        bytearray(scalene.scalene_config.NEWLINE_TRIGGER_LENGTH)
 
     @staticmethod
     def update_profiled() -> None:
@@ -368,7 +375,7 @@ class Scalene:
             # This can happen when Scalene shuts down.
             return None
         except Exception as e:
-            print(f"{Scalene.__error_message}:\n", e)
+            print(f"{Scalene.__error_message}:\n", e, file=sys.stderr)
             traceback.print_exc()
             return None
 
@@ -645,7 +652,7 @@ class Scalene:
         if sys.platform == "win32":
             if arguments.memory:
                 print(
-                    f"Scalene warning: Memory profiling is not currently supported for Windows."
+                    f"Scalene warning: Memory profiling is not currently supported for Windows.", file=sys.stderr
                 )
                 arguments.memory = False
 
@@ -691,6 +698,7 @@ class Scalene:
                 "memory",
                 "cli",
                 "web",
+                "html",
                 "no_browser",
                 "reduced_profile",
             ]:
@@ -809,7 +817,7 @@ class Scalene:
     def output_profile(program_args: Optional[List[str]] = None) -> bool:
         """Output the profile. Returns true iff there was any info reported the profile."""
         # sourcery skip: inline-immediately-returned-variable
-        # print(Scalene.flamegraph_format(Scalene.__stats.stacks))
+        # print(flamegraph_format(Scalene.__stats.stacks))       
         if Scalene.__args.json:
             json_output = Scalene.__json.output_profiles(
                 Scalene.__program_being_profiled,
@@ -1013,16 +1021,21 @@ class Scalene:
 
         main_thread_frame = new_frames[0][0]
 
-        if Scalene.__args.stacks:
-            add_stack(
-                main_thread_frame, Scalene.should_trace, Scalene.__stats.stacks
-            )
-
         average_python_time = python_time / total_frames
         average_c_time = c_time / total_frames
         average_gpu_time = gpu_time / total_frames
         average_cpu_time = (python_time + c_time) / total_frames
 
+        if Scalene.__args.stacks:
+            add_stack(
+                main_thread_frame,
+                Scalene.should_trace,
+                Scalene.__stats.stacks,
+                average_python_time,
+                average_c_time,
+                average_cpu_time
+            )
+
         # First, handle the main thread.
         Scalene.enter_function_meta(main_thread_frame, Scalene.__stats)
         fname = Filename(main_thread_frame.f_code.co_filename)
@@ -1049,7 +1062,12 @@ class Scalene:
         for (frame, tident, orig_frame) in new_frames:
             if frame == main_thread_frame:
                 continue
-            add_stack(frame, Scalene.should_trace, Scalene.__stats.stacks)
+            add_stack(frame,
+                      Scalene.should_trace,
+                      Scalene.__stats.stacks,
+                      average_python_time,
+                      average_c_time,
+                      average_cpu_time)
 
             # In a thread.
             fname = Filename(frame.f_code.co_filename)
@@ -1237,7 +1255,7 @@ class Scalene:
                 _alloc_time,
                 action,
                 count,
-                _python_fraction,
+                python_fraction,
                 pointer,
                 fname,
                 lineno,
@@ -1249,6 +1267,7 @@ class Scalene:
                 stats.current_footprint += count
                 if stats.current_footprint > stats.max_footprint:
                     stats.max_footprint = stats.current_footprint
+                    stats.max_footprint_python_fraction = python_fraction
                     stats.max_footprint_loc = (fname, lineno)
             else:
                 assert action in [
@@ -1309,7 +1328,7 @@ class Scalene:
             ) = item
 
             is_malloc = action == Scalene.MALLOC_ACTION
-            if is_malloc and count == NEWLINE_TRIGGER_LENGTH + 1:
+            if is_malloc and count == scalene.scalene_config.NEWLINE_TRIGGER_LENGTH + 1:
                 with Scalene.__invalidate_mutex:
                     last_file, last_line = Scalene.__invalidate_queue.pop(0)
 
@@ -1498,8 +1517,9 @@ class Scalene:
         if not Scalene.__args.profile_all:
             for n in sysconfig.get_scheme_names():
                 for p in sysconfig.get_path_names():
+                    the_path = sysconfig.get_path(p, n)
                     libdir = str(
-                        pathlib.Path(sysconfig.get_path(p, n)).resolve()
+                        pathlib.Path(the_path).resolve()
                     )
                     if libdir in resolved_filename:
                         return False
@@ -1556,7 +1576,8 @@ class Scalene:
             print(
                 "ERROR: Do not try to invoke `start` if you have not called Scalene using one of the methods\n"
                 "in https://github.com/plasma-umass/scalene#using-scalene\n"
-                "(The most likely issue is that you need to run your code with `scalene`, not `python`)."
+                "(The most likely issue is that you need to run your code with `scalene`, not `python`).",
+                file=sys.stderr
             )
             sys.exit(1)
         Scalene.__stats.start_clock()
@@ -1578,6 +1599,7 @@ class Scalene:
             pywhere.set_scalene_done_true()
 
 
+
         Scalene.disable_signals()
         Scalene.__stats.stop_clock()
         if Scalene.__args.outfile:
@@ -1712,9 +1734,9 @@ class Scalene:
             exit_status = se.code if type(se.code) == int else 1
         except KeyboardInterrupt:
             # Cleanly handle keyboard interrupts (quits execution and dumps the profile).
-            print("Scalene execution interrupted.")
+            print("Scalene execution interrupted.", file=sys.stderr)
         except Exception as e:
-            print(f"{Scalene.__error_message}:\n", e)
+            print(f"{Scalene.__error_message}:\n", e, file=sys.stderr)
             traceback.print_exc()
             exit_status = 1
         finally:
@@ -1734,9 +1756,41 @@ class Scalene:
             did_output = Scalene.output_profile(left)
             if not did_output:
                 print(
-                    "Scalene: Program did not run for long enough to profile."
+                    "Scalene: The specified code did not run for long enough to profile.",
+                    file=sys.stderr
                 )
-
+                # Print out hints to explain why the above message may have been printed.
+                if not Scalene.__args.profile_all:
+                    # if --profile-all was not specified, suggest it
+                    # as a way to profile otherwise excluded code
+                    # (notably Python libraries, which are excluded by
+                    # default).
+                    print(
+                        "By default, Scalene only profiles code in the file executed and its subdirectories.",
+                        file=sys.stderr
+                        )
+                    print(
+                        "To track the time spent in all files, use the `--profile-all` option.",
+                        file=sys.stderr
+                    )
+                elif Scalene.__args.profile_only or Scalene.__args.profile_exclude:
+                    # if --profile-only or --profile-exclude were
+                    # specified, suggest that the patterns might be
+                    # excluding too many files. Collecting the
+                    # previously filtered out files could allow
+                    # suggested fixes (as in, remove foo because it
+                    # matches too many files).
+                    print("The patterns used in `--profile-only` or `--profile-exclude` may be filtering out too many files.",
+                          file=sys.stderr
+                          )
+                else:
+                    # if none of the above cases hold, indicate that
+                    # Scalene can only profile code that runs for at
+                    # least one second or allocates some threshold
+                    # amount of memory.
+                    print("Scalene can only profile code that runs for at least one second or allocates at least 10MB.",
+                          file=sys.stderr
+                        )
             if not (
                 did_output
                 and Scalene.__args.web
@@ -1756,7 +1810,8 @@ class Scalene:
 
                 port = ScaleneJupyter.find_available_port(8181, 9000)
                 if not port:
-                    print("Scalene error: could not find an available port.")
+                    print("Scalene error: could not find an available port.",
+                          file=sys.stderr)
                 else:
                     ScaleneJupyter.display_profile(
                         port, Scalene.__profiler_html
@@ -1778,15 +1833,13 @@ class Scalene:
                         # url = f"file:///{output_fname}"
                         # webbrowser.open(url)
                         # show_browser(output_fname, SCALENE_PORT, Scalene.__orig_python)
-                        if True:
-                            dir = os.path.dirname(__file__)
-                            import subprocess
-                            subprocess.Popen([Scalene.__orig_python,
-                                              f"{dir}{os.sep}launchbrowser.py",
-                                              output_fname,
-                                              str(SCALENE_PORT)],
-                                             stdout=subprocess.DEVNULL,
-                                             stderr=subprocess.DEVNULL)
+                        dir = os.path.dirname(__file__)
+                        subprocess.Popen([Scalene.__orig_python,
+                                          f"{dir}{os.sep}launchbrowser.py",
+                                          output_fname,
+                                          str(scalene.scalene_config.SCALENE_PORT)],
+                                         stdout=subprocess.DEVNULL,
+                                         stderr=subprocess.DEVNULL)
                     # Restore them.
                     os.environ.update(
                         {
@@ -1857,7 +1910,8 @@ class Scalene:
         if not Scalene.__initialized:
             print(
                 "ERROR: Do not try to manually invoke `run_profiler`.\n"
-                "To invoke Scalene programmatically, see the usage noted in https://github.com/plasma-umass/scalene#using-scalene"
+                "To invoke Scalene programmatically, see the usage noted in https://github.com/plasma-umass/scalene#using-scalene",
+                file=sys.stderr
             )
             sys.exit(1)
         if sys.platform != "win32":
@@ -1882,12 +1936,14 @@ class Scalene:
                 # If running in the background, print the PID.
                 if os.getpgrp() != os.tcgetpgrp(sys.stdout.fileno()):
                     # In the background.
-                    print(f"Scalene now profiling process {os.getpid()}")
+                    print(f"Scalene now profiling process {os.getpid()}", file=sys.stderr)
                     print(
-                        f"  to disable profiling: python3 -m scalene.profile --off --pid {os.getpid()}"
+                        f"  to disable profiling: python3 -m scalene.profile --off --pid {os.getpid()}",
+                        file=sys.stderr
                     )
                     print(
-                        f"  to resume profiling:  python3 -m scalene.profile --on  --pid {os.getpid()}"
+                        f"  to resume profiling:  python3 -m scalene.profile --on  --pid {os.getpid()}",
+                        file=sys.stderr
                     )
         Scalene.__stats.clear_all()
         sys.argv = left
@@ -2009,13 +2065,13 @@ class Scalene:
                     except Exception as ex:
                         template = "Scalene: An exception of type {0} occurred. Arguments:\n{1!r}"
                         message = template.format(type(ex).__name__, ex.args)
-                        print(message)
-                        print(traceback.format_exc())
+                        print(message, file=sys.stderr)
+                        print(traceback.format_exc(), file=sys.stderr)
             except (FileNotFoundError, IOError):
                 if progs:
-                    print(f"Scalene: could not find input file {prog_name}")
+                    print(f"Scalene: could not find input file {prog_name}", file=sys.stderr)
                 else:
-                    print("Scalene: no input file specified.")
+                    print("Scalene: no input file specified.", file=sys.stderr)
                 sys.exit(1)
         except SystemExit as e:
             exit_status = e.code if type(e.code) == int else 1
@@ -2023,7 +2079,7 @@ class Scalene:
         except StopJupyterExecution:
             pass
         except Exception:
-            print("Scalene failed to initialize.\n" + traceback.format_exc())
+            print("Scalene failed to initialize.\n" + traceback.format_exc(), file=sys.stderr)
             sys.exit(1)
         finally:
             with contextlib.suppress(Exception):
