@@ -40,7 +40,7 @@ class ScaleneStatistics:
         self.alloc_samples: int = 0
 
         #  full stacks taken during CPU samples, together with number of hits
-        self.stacks: Dict[Tuple[Any], int] = defaultdict(int)
+        self.stacks: Dict[Tuple[Any], Any] = defaultdict(None)
 
         #   CPU samples for each location in the program
         #   spent in the interpreter
@@ -161,6 +161,7 @@ class ScaleneStatistics:
 
         # the peak memory footprint
         self.max_footprint: float = 0.0
+        self.max_footprint_python_fraction : float = 0
         self.max_footprint_loc: Optional[Tuple[Filename, LineNumber]] = None
         # memory footprint samples (time, footprint)
         self.memory_footprint_samples: List[List[float]] = []
