@@ -13,8 +13,9 @@ import webbrowser
 from jinja2 import Environment, FileSystemLoader
 from types import CodeType, FrameType
 from typing import Any, Callable, Dict, List, Optional, Tuple, cast
+
 from scalene.scalene_statistics import Filename, LineNumber
-from scalene.scalene_version import scalene_version, scalene_date
+from scalene.scalene_config import scalene_version, scalene_date
 
 # These are here to simplify print debugging, a la C.
 class LineNo:
@@ -41,7 +42,10 @@ __FILE__ = FileName()
 def add_stack(
     frame: FrameType,
     should_trace: Callable[[Filename, str], bool],
-    stacks: Dict[Any, int],
+    stacks: Dict[Any, Any],
+    python_time: float,
+    c_time: float,
+    cpu_samples: float
 ) -> None:
     """Add one to the stack starting from this frame."""
     stk: List[Tuple[str, str, int]] = list()
@@ -50,7 +54,15 @@ def add_stack(
         if should_trace(Filename(f.f_code.co_filename), f.f_code.co_name):
             stk.insert(0, (f.f_code.co_filename, f.f_code.co_name, f.f_lineno))
         f = f.f_back
-    stacks[tuple(stk)] += 1
+    if tuple(stk) not in stacks:
+        stacks[tuple(stk)] = (1, python_time, c_time, cpu_samples)
+    else:
+        (prev_count, prev_python_time, prev_c_time, prev_cpu_samples) = stacks[tuple(stk)]
+        stacks[tuple(stk)] = (prev_count + 1,
+                              prev_python_time + python_time,
+                              prev_c_time + c_time,
+                              prev_cpu_samples + cpu_samples)
+        # stacks[tuple(stk)] += 1
 
 
 def on_stack(
@@ -97,18 +109,22 @@ def get_fully_qualified_name(frame: FrameType) -> Filename:
     return fn_name
 
 
-def flamegraph_format(stacks: Dict[Tuple[Any], int]) -> str:
+def flamegraph_format(stacks: Dict[Tuple[Any], Any]) -> str:
     """Converts stacks to a string suitable for input to Brendan Gregg's flamegraph.pl script."""
     output = ""
     for stk in stacks.keys():
         for item in stk:
             (fname, fn_name, lineno) = item
             output += f"{fname} {fn_name}:{lineno};"
-        output += " " + str(stacks[stk])
+        output += " " + str(stacks[stk][0])
         output += "\n"
     return output
 
 
+def read_file_content(directory: str, subdirectory: str, filename: str) -> str:
+    file_path = os.path.join(directory, subdirectory, filename)
+    return pathlib.Path(file_path).read_text()
+
 def generate_html(profile_fname: Filename, output_fname: Filename) -> None:
     """Apply a template to generate a single HTML payload containing the current profile."""
 
@@ -117,14 +133,21 @@ def generate_html(profile_fname: Filename, output_fname: Filename) -> None:
         profile_file = pathlib.Path(profile_fname)
         profile = profile_file.read_text()
     except FileNotFoundError:
-        return
+        assert profile_fname == "demo"
+        profile = ""
+        # return
 
     # Load the GUI JavaScript file.
     scalene_dir = os.path.dirname(__file__)
-    gui_fname = os.path.join(scalene_dir, "scalene-gui", "scalene-gui.js")
-    gui_file = pathlib.Path(gui_fname)
-    gui_js = gui_file.read_text()
 
+    file_contents = {
+        'scalene_gui_js_text': read_file_content(scalene_dir, "scalene-gui", "scalene-gui.js"),
+        'prism_css_text': read_file_content(scalene_dir, "scalene-gui", "prism.css"),
+        'prism_js_text': read_file_content(scalene_dir, "scalene-gui", "prism.js"),
+        'tablesort_js_text': read_file_content(scalene_dir, "scalene-gui", "tablesort.js"),
+        'tablesort_number_js_text': read_file_content(scalene_dir, "scalene-gui", "tablesort.number.js")
+    }
+    
     # Put the profile and everything else into the template.
     environment = Environment(
         loader=FileSystemLoader(os.path.join(scalene_dir, "scalene-gui"))
@@ -132,7 +155,11 @@ def generate_html(profile_fname: Filename, output_fname: Filename) -> None:
     template = environment.get_template("index.html.template")
     rendered_content = template.render(
         profile=profile,
-        gui_js=gui_js,
+        gui_js=file_contents['scalene_gui_js_text'],
+        prism_css=file_contents['prism_css_text'],
+        prism_js=file_contents['prism_js_text'],
+        tablesort_js=file_contents['tablesort_js_text'],
+        tablesort_number_js=file_contents['tablesort_number_js_text'],
         scalene_version=scalene_version,
         scalene_date=scalene_date,
     )
@@ -145,7 +172,7 @@ def generate_html(profile_fname: Filename, output_fname: Filename) -> None:
         pass
 
 
-def start_server(port, directory):
+def start_server(port: int, directory: str) -> None:
     try:
         handler = http.server.SimpleHTTPRequestHandler
         with socketserver.TCPServer(("", port), handler) as httpd:
@@ -156,7 +183,7 @@ def start_server(port, directory):
         # print(f"Port {port} is already in use. Please try a different port.")
         pass
 
-def show_browser(file_path, port, orig_python='python3'):
+def show_browser(file_path: str, port: int, orig_python : str ='python3') -> None:
     temp_dir = tempfile.gettempdir()
 
     # Copy file to the temporary directory
@@ -182,3 +209,4 @@ def show_browser(file_path, port, orig_python='python3'):
         pass
     finally:
         os.chdir(curr_dir)
+
